/*
* helpers.cpp
*
* This file contains static helper functions
*
* Copyright 2010 Tobias Wirtl, HBM <tobias.wirtl@hbm.com>
*
* The code contained herein is licensed under the GNU General Public
* License. You may obtain a copy of the GNU General Public License
* Version 2 or later at the following locations:
*
* http://www.opensource.org/licenses/gpl-license.html
* http://www.gnu.org/copyleft/gpl.html
*/


#include "helpers.h"
#include <iostream>
#include <math.h>
#include <sstream>


Helpers::Helpers()
{
}

void Helpers::splitStr(const string &toSplit, vector<string> *target, char seperator)
{
    unsigned int begin = 0;
    unsigned int i;

    for (i = 0; i < toSplit.size(); i++)
    {
        if (toSplit[i] == seperator || i + 1 ==toSplit.size())
        {
            if (i + 1 == toSplit.size()) i++;
            target->push_back(toSplit.substr(begin,i-begin));
            begin = i + 1;
        }
    }
}



int Helpers::strToInt(const string &aString)
{
    int result = 0;
    int digit;
    unsigned int i;

    for (i = 0; i < aString.size(); i++)
    {
        if (aString[i] > 47 && aString[i] < 58)
        {
            digit = aString[i]-48;
            result += digit*pow(10,aString.size()-(i+1));
        }
        else
            cout<<"Warning: strToInt "<<aString[i]<<" -> not an integer value";
    }
    return result;
}


string Helpers::intToStr(const int &aInt)
{
    string result;
    stringstream tmp;
    tmp<<aInt;
    tmp>>result;
    return result;
}


int Helpers::hexStrToDecInt(const string &aString)
{
    int result = 0;
    int digit;
    unsigned int i;

    for (i = 0; i < aString.size(); i++)
    {
        if ((aString[i] > 47 && aString[i] < 58) || (aString[i] > 96 && aString[i] < 103))
        {
            if (aString[i] < 58 )
                digit = aString[i]-48;
            else
                digit = aString[i]-87;

            result += digit*pow(16,aString.size()-(i+1));
        }
        else
            cout<<"Warning: hexStrToDecInt "<<aString[i]<<" -> not a hex value";
    }
    return result;
}

int Helpers::binStrToDecInt(const string &aString)
{
    int result = 0;
    int digit;
    unsigned int i;

    for (i = 0; i < aString.size(); i++)
    {
        if (aString[i] > 47 && aString[i] < 50)
        {
            digit = aString[i]-48;
            result += digit*pow(2,aString.size()-(i+1));
        }
        else
            cout<<"Warning: binStrToDecInt "<<aString[i]<<" -> not a binary value";
    }
    return result;
}

string Helpers::intToBinaryStr(unsigned int i)
{
    string res;
    while (i > 0)
    {
        if (i%2)
            res = '1' + res;
        else
            res = '0' + res;
        i = i / 2;
    }

    return res;
}

string Helpers::intToHexStr(unsigned int i, unsigned int fill)
{
    string res;
    char hex[] = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};

    while (i > 0)
    {
        res = hex[i % 16] + res;
        i = i / 16;
    }

    while(res.size() < fill)
        res = '0' + res;

    return "0x" + res;
}

string Helpers::build2Complement(const string &binaryValues)
{
    bool added = false;
    int i;
    string result = binaryValues;
    for(i = binaryValues.length()-1;i >= 0; i--)
    {
        //invert and add one
        result[i] == '1' ? result[i] = '0' : result[i] = '1';
        if (!added)
        {
            if (result[i] == '1')
                result[i] = '0';
            else
            {
                result[i] = '1';
                added=true;
            }
        }
    }
    return result;
}

void Helpers::strToLower(string &aString)
{
    unsigned int i;
    for(i = 0; i < aString.length();i++)
        aString[i] = std::tolower(aString[i]);
}

string Helpers::replace(const string &sourceStr, const string &oldStr, const string &newStr)
{
    string tmp = sourceStr;
    int i;
    i = sourceStr.find(oldStr.c_str());
    while (i > -1)
    {
        tmp.replace (i, oldStr.length(),newStr, 0, newStr.length() );
        i = tmp.find(oldStr.c_str(),i+newStr.length());
    }
    return tmp;
}
